create or replace package moats as

   -- MOATS v1.0 (c) Tanel Poder & Adrian Billington, 2010

   -- Pipelined function to show TOP instance summary activity. The refresh rate
   -- defines how many ASH/SYSSTAT samples to take and is roughly equivalent to 
   -- the number of seconds between screen refreshes...
   -- --------------------------------------------------------------------------
	function top ( 
				p_refresh_rate in integer default 10
				) return moats_output_ntt pipelined;

   -- Pipelined function to return dynamic samples, queries, aggregations etc
   -- of active session data. The refresh rate defines (in seconds) how many 
   -- ASH samples to take before running the query components...
   -- --------------------------------------------------------------------------
	function ash ( 
				p_refresh_rate in integer  default 10,
				p_select       in varchar2 default null,
				p_where        in varchar2 default null,
				p_group_by	   in varchar2 default null,
				p_order_by     in varchar2 default null
				) return moats_output_ntt pipelined;

   -- Until 11g you can't bind UDTs into DBMS_SQL cursors, so this is an
   -- internal workaround. Can also be used to query all of the collected ASH
   -- samples. Note that gc_all_rows returns all ASH records between the two
   -- snapshots and gc_deltas returns only the lower and upper snapshots... 
   -- --------------------------------------------------------------------------

   gc_all_rows constant pls_integer := 0;
   gc_deltas   constant pls_integer := 1; 

	function get_ash ( 
				p_lower_snap in pls_integer default null,
				p_upper_snap in pls_integer default null,
            p_return_set in pls_integer default moats.gc_all_rows
				) return moats_ash_ntt pipelined;

   -- Constants, get/set for moats parameters such as polling rate, ASH size...
   -- --------------------------------------------------------------------------
   gc_polling_rate  constant pls_integer := 0;
   gc_ash_threshold constant pls_integer := 1;
 
	procedure set_parameter( 
             p_parameter_code  in pls_integer,
             p_parameter_value in integer 
				 );

	function get_parameter(
            p_parameter_code in pls_integer
            ) return integer;

   procedure restore_default_parameters;

   -- Debugging...
   -- --------------------------------------------------------------------------
   procedure show_snaps;

end moats;
/
